/*
 * Reference http://forum.cocos.com/t/topic/44304/10
 * @Description: 二维码组件
 */
const { ccclass, property } = cc._decorator;
import { QRCode,QRErrorCorrectLevel } from "../Third/QRCode";

@ccclass
export default class QRCodeComponent extends cc.Component {

    @property({
        tooltip : "二维码内容"
    })
	private content: string = "";

	getContent() {
		return this.content;
	}

	@property({
		tooltip:"白边的宽度,为缩放后的实际宽度"
	})
	padding: number = 10;
 
	get noneNode() {
		if (!this._isNoneNodeInited) {
			this._noneNode = cc.find("none", this.node);
			this._isNoneNodeInited = true;
		}			
		return this._noneNode;
	}

	private _isNoneNodeInited: boolean = false;
	private _noneNode: cc.Node;
	private _qrCodeNode: cc.Node;
	onLoad() {
		this.noneNode.active = false;
		this._createQrCodeNode();
		this._resizeQrCodeSize();
		if (this.content != null) {
			this._setContent();
		}
	}
	private _createQrCodeNode() {
		let node = new cc.Node("qrCodeInner");
		node.width = this.node.width;
		node.height = this.node.height;
		node.parent = this.node;
		this._qrCodeNode = node;
		this._qrCodeNode.addComponent(cc.Graphics);
	}

	private _resizeQrCodeSize() {
		let realWidth = this.node.width - this.padding * 2;
		const minWidth = 256;
		if (realWidth < minWidth) {
			let scale = realWidth/minWidth;
			this._qrCodeNode.width = this.node.width / scale;// minWidth + Math.floor(this.padding*2* scale);
			this._qrCodeNode.setScale(scale);
		}
		this._qrCodeNode.height = this._qrCodeNode.width;
	}

	private _refreshQrContent() {
		if (this._qrCodeNode == null) {
			this._createQrCodeNode();
		}
		let qrCodeNode = this._qrCodeNode;
		let originSize = qrCodeNode.width;
		let padding = Math.floor( this.padding / qrCodeNode.scale);
		let size = originSize - padding * 2;
		let ctx = qrCodeNode.getComponent(cc.Graphics);
		ctx.clear();
		// 白底
		ctx.fillColor = cc.Color.WHITE;
		ctx.rect(0,0,originSize,originSize);
		ctx.fill();
		ctx.fillColor = cc.Color.BLACK;
		if (this.content) {
			var qrcode = new QRCode(-1, QRErrorCorrectLevel.L);
			qrcode.addData(this.content);
			qrcode.make();
			let num = qrcode.getModuleCount();
			// compute tileW/tileH based on node width and height
			var tileW = size / num;
			var tileH = size / num;
			// draw in the Graphics
			for (var row = 0; row < num; row++) {
				for (var col = 0; col < num; col++) {
					if (qrcode.isDark(row, col)) {
						// cc.log(row, col)
						// ctx.fillColor = cc.Color.BLACK;
						var w = (Math.ceil((col + 1) * tileW) - Math.floor(col * tileW));
						var h = (Math.ceil((row + 1) * tileW) - Math.floor(row * tileW));
						let x = Math.round(col * tileW)+padding;
						let y = size - tileH - Math.round(row * tileH)+padding;
						ctx.rect(x,
							y,
							w, h);
						ctx.fill();
					} else {
						// ctx.fillColor = cc.Color.WHITE;
					}
				}
			}
		}
	}

	private _setContent() {
		this._refreshQrContent();
		let showQrCode = !!this.content;
		if (this.noneNode) {
			this.noneNode.active = !showQrCode;
		}
		this._qrCodeNode.active = showQrCode;
	}
	public setContent(value: string) {
		// 未改变，不需要更新
		if (this.content === value) {
			return;
		}
		this.content = value;
		this._setContent();
	}
}